/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <fontconfig.h>
#include <dlog.h>
#include <Elementary.h>
#include <string.h>

#include "fontconfig_matched_font_screen.h"
#include "tizen_open_source_fontconfig.h"

#define LABEL_BUFFER_SIZE 1024
static bool by_capability = false;
static char* otlayout = NULL;

static void _show_fonts_log(appdata_s *ad, FcFontSet *fs)
{
    if (!ad || !fs) {
        dlog_print(DLOG_ERROR, LOG_TAG, "NULL pointer received");
        return;
    }

    int i = 0;

    FcPattern *font = NULL;
    FcChar8 *s = NULL;

    if (by_capability) {
        if (otlayout != NULL) {
            int nr = 0;
            for (i = 0; i < fs->nfont; i++) {
                font = fs->fonts[i];
                s = FcNameUnparse(font);
                char *position = strstr((char*)s, otlayout);
                if (position != NULL) dlog_print(DLOG_DEBUG, LOG_TAG, "Font %d >> %s", ++nr, s);
                free(s);
            }
        }
        return;
    }

    dlog_print(DLOG_DEBUG, LOG_TAG, "Total fonts: %d", fs->nfont);
    for (i = 0; i < fs->nfont; i++) {
        font = fs->fonts[i];
        s = FcNameUnparse(font);
        dlog_print(DLOG_DEBUG, LOG_TAG, "Font %d >> %s", (i + 1), s);
        free(s);
    }
}

static void _fetch_font_full_name(FcFontSet *fs, FcPattern *font_object, Evas_Object *fonts_list, int nr)
{
    FcChar8 *family_name = NULL;
    FcChar8 *full_name = NULL;
    Evas_Object *label = NULL;
    char buffer[LABEL_BUFFER_SIZE];

    // Try to fetch font full name. In most cases it's font family name
    // with added style description (Regular, Medium and so).
    // If not possible try to fetch font family name
    if (FcPatternGetString(font_object, FC_FULLNAME, 0, &full_name) == FcResultMatch
            || FcPatternGetString(font_object, FC_FAMILY, 0, &family_name) == FcResultMatch) {
        memset(buffer, '\0', LABEL_BUFFER_SIZE);
        // Prepare buffer with label content:
        // Font description opening tags + font full name + closing tags
        snprintf(buffer, LABEL_BUFFER_SIZE - 1, "<align=left><style=bottom>"
                 "<font_size=28><font=\'%s\'>%s</font_name></font_size></style></align>",
                 // If full name not fetched then use family name.
                 // Probably text will not be displayed using specified font
                 // but font name will be visible
                 full_name ? full_name : family_name, full_name ? full_name : family_name);
        dlog_print(DLOG_DEBUG, LOG_TAG, "Adding %s to list...",
                   full_name ? full_name : family_name);

        label = elm_label_add(fonts_list);
        elm_object_text_set(label, buffer);
        elm_list_item_append(fonts_list, NULL, label, NULL, NULL, NULL);
    } else {
        // If failed to fetch both full name and family name then it is not
        // possible to display font information on list. Error message is
        // sent to dlog.
        dlog_print(DLOG_ERROR, LOG_TAG, "Failed to fetch font full name");
    }
}

static void _show_fonts_ui(appdata_s *ad, FcFontSet *fs)
{
    if (!ad || !fs) {
        dlog_print(DLOG_ERROR, LOG_TAG, "NULL pointer received");
        return;
    }

    // Fontconfig related variables
    FcPattern *font_object = NULL;

    // Elementary related variables
    Evas_Object *fonts_list = NULL;
    Evas_Object *label = NULL;

    // Create elm_list for font presentation
    fonts_list = elm_list_add(ad->win);

    if (!fonts_list) {
        dlog_print(DLOG_ERROR, LOG_TAG, "Failed to add new list");
        return;
    }

    elm_scroller_policy_set(fonts_list, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);
    evas_object_show(fonts_list);
    elm_naviframe_item_push(ad->navi, ad->app_name, NULL, NULL, fonts_list, NULL);

    char buffer[LABEL_BUFFER_SIZE];

    if (fs->nfont == 0) {
        memset(buffer, '\0', LABEL_BUFFER_SIZE);
        label = elm_label_add(fonts_list);
        elm_object_text_set(label, buffer);
        elm_list_item_append(fonts_list, "No font has been found", label, NULL, NULL, NULL);
    }

    // Iterate on table of search results
    if (by_capability) {
        FcChar8 *s = NULL;
        if (otlayout != NULL) {
            int nr = 0;
            for (int i = 0; i < fs->nfont; i++) {
                font_object = fs->fonts[i];

                s = FcNameUnparse(font_object);
                char *position = strstr((char*)s, otlayout);
                if (position != NULL) _fetch_font_full_name(fs, font_object, fonts_list, nr);
                free(s);
            }
        }
    } else {
        dlog_print(DLOG_DEBUG, LOG_TAG, "Total fonts: %d", fs->nfont);
        for (int i = 0; i < fs->nfont; i++) {
            font_object = fs->fonts[i];

            _fetch_font_full_name(fs, font_object, fonts_list, i+1);
        }
    }

    // Fonts should be only displayed without possibility of selection
    // (no action after clicking list item)
    elm_list_select_mode_set(fonts_list, ELM_OBJECT_SELECT_MODE_NONE);
    elm_list_go(fonts_list);
}

void show_fonts_by_weight(appdata_s *ad, const char *weight)
{
    if (!ad || !weight) {
        dlog_print(DLOG_ERROR, LOG_TAG, "NULL pointer received");
        return;
    }

    int fontweight = 0;

    // Font weight is stored in fontconfig as integer value
    if (!strcmp(weight, "Regular"))
        fontweight = FC_WEIGHT_REGULAR;
    else if (!strcmp(weight, "Medium"))
        fontweight = FC_WEIGHT_MEDIUM;
    else if (!strcmp(weight, "Semibold"))
        fontweight = FC_WEIGHT_SEMIBOLD;

    FcPattern *pat = NULL;
    FcFontSet *fontlist = NULL;
    FcObjectSet *os = NULL;
    FcConfig *config;
    FcBool result;

    // Initialize Fontconfig library
    result = FcInit();

    if (!result) {
        dlog_print(DLOG_ERROR, LOG_TAG, "Failed to initialize Fontconfig library");
        return;
    }

    // Fetch current fontoconfig configuration
    config = FcConfigGetCurrent();
    // Disable periodical fonts rescanning
    FcConfigSetRescanInterval(config, 0);

    // Create search pattern (object that contains desired font parameters)
    pat = FcPatternCreate();

    if (!pat) {
        dlog_print(DLOG_ERROR, LOG_TAG, "Failed to create search pattern");
        return;
    }

    // Add constrain (with integer type of value) to search pattern:
    // font weight equal to given parameter
    FcPatternAddInteger(pat, FC_WEIGHT, fontweight);
    // Create object set - template object that describes font
    // In this case font in returned search result list will be described by:
    // Font family name (FC_FAMILY)
    // Supported languages (FC_LANG)
    // Font weight (FC_WEIGHT)
    // Font style (FC_STYLE)
    // Font additional capabilities (FC_CAPABILITY)
    // Font full name (FC_FULLNAME, commonly it's concatenation of FC_FAMILY and FC_STYLE)
    os = FcObjectSetBuild(FC_FAMILY, FC_LANG, FC_WEIGHT, FC_STYLE, FC_CAPABILITY,
                          FC_FULLNAME, (char *)0);
    // Start font search and return list of results
    fontlist = FcFontList(config, pat, os);

    if (!fontlist)
        dlog_print(DLOG_ERROR, LOG_TAG, "No search result returned.");

    // print informations about each font found for debugging purposes
    _show_fonts_log(ad, fontlist);
    // create list of font names printed with use of these fonts
    _show_fonts_ui(ad, fontlist);

    // List of fonts no more needed - destroy it
    if (fontlist) {
        FcFontSetDestroy(fontlist);
        fontlist = NULL;
    }
}

void show_fonts_by_lang(appdata_s *ad, const char *lang)
{
    // For detailed code description please see similar function
    // show_fonts_by_weight()

    if (!ad || !lang) {
        dlog_print(DLOG_ERROR, LOG_TAG, "NULL pointer received");
        return;
    }

    FcPattern *pat = NULL;
    FcFontSet *fontlist = NULL;
    FcObjectSet *os = NULL;
    FcConfig *config;
    FcBool result;

    result = FcInit();

    if (!result) {
        dlog_print(DLOG_ERROR, LOG_TAG, "Failed to initialize Fontconfig library");
        return;
    }

    config = FcConfigGetCurrent();
    FcConfigSetRescanInterval(config, 0);

    pat = FcPatternCreate();

    if (!pat) {
        dlog_print(DLOG_ERROR, LOG_TAG, "Failed to create search pattern");
        return;
    }

    // Add constrain (with string type of value) to search patern:
    // font supports language defined by lang variable
    FcPatternAddString(pat, FC_LANG, (FcChar8 *)lang);
    os = FcObjectSetBuild(FC_FAMILY, FC_LANG, FC_WEIGHT, FC_STYLE, FC_WIDTH, FC_SIZE,
                          FC_CAPABILITY, FC_FULLNAME, (char *)0);
    fontlist = FcFontList(config, pat, os);

    _show_fonts_log(ad, fontlist);
    _show_fonts_ui(ad, fontlist);

    if (fontlist) {
        FcFontSetDestroy(fontlist);
        fontlist = NULL;
    }
}

void show_fonts_by_style(appdata_s *ad, const char *style)
{
    // For detailed code description please see similar function
    // show_fonts_by_weight()

    if (!ad || !style) {
        dlog_print(DLOG_ERROR, LOG_TAG, "NULL pointer received");
        return;
    }

    FcPattern *pat = NULL;
    FcFontSet *fontlist = NULL;
    FcObjectSet *os = NULL;
    FcConfig *config;
    FcBool result;

    result = FcInit();

    if (!result) {
        dlog_print(DLOG_ERROR, LOG_TAG, "Failed to initialize Fontconfig library");
        return;
    }

    config = FcConfigGetCurrent();
    FcConfigSetRescanInterval(config, 0);

    pat = FcPatternCreate();

    if (!pat) {
        dlog_print(DLOG_ERROR, LOG_TAG, "Failed to create search pattern");
        return;
    }

    // Add constrain (with string type of value) to search pattern:
    // font with given style
    FcPatternAddString(pat, FC_STYLE, (FcChar8 *)style);
    os = FcObjectSetBuild(FC_FAMILY, FC_LANG, FC_WEIGHT, FC_STYLE, FC_WIDTH, FC_SIZE,
                          FC_CAPABILITY, FC_FULLNAME, (char *)0);
    fontlist = FcFontList(config, pat, os);

    _show_fonts_log(ad, fontlist);
    _show_fonts_ui(ad, fontlist);

    if (fontlist) {
        FcFontSetDestroy(fontlist);
        fontlist = NULL;
    }
}

void show_fonts_by_capability(appdata_s *ad, const char *caps)
{
    by_capability = true;
    int caps_len = strlen(caps);
    otlayout = (char*)calloc(1, caps_len+2);
    strncpy(otlayout, caps, caps_len);
    // For detailed code description please see similar function
    // show_fonts_by_weight()

    if (!ad || !caps) {
        dlog_print(DLOG_ERROR, LOG_TAG, "NULL pointer received");
        return;
    }

    FcPattern *pat = NULL;
    FcFontSet *fontlist = NULL;
    FcObjectSet *os = NULL;
    FcConfig *config;
    FcBool result;

    result = FcInit();

    if (!result) {
        dlog_print(DLOG_ERROR, LOG_TAG, "Failed to initialize Fontconfig library");
        return;
    }

    config = FcConfigGetCurrent();
    FcConfigSetRescanInterval(config, 0);

    pat = FcPatternCreate();

    if (!pat) {
        dlog_print(DLOG_ERROR, LOG_TAG, "Failed to create search pattern");
        return;
    }

    // Add constrain (with string type of value) to search patern:
    // font capability contains given value

    os = FcObjectSetBuild(FC_FAMILY, FC_LANG, FC_WEIGHT, FC_STYLE, FC_WIDTH, FC_SIZE,
                          FC_CAPABILITY, FC_FULLNAME, (char *)0);
    fontlist = FcFontList(config, pat, os);

    _show_fonts_log(ad, fontlist);
    _show_fonts_ui(ad, fontlist);

    if (fontlist) {
        FcFontSetDestroy(fontlist);
        fontlist = NULL;
    }
}
