/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "fontconfig_value_selection_screen.h"
#include "fontconfig_matched_font_screen.h"

// Values for all lists on sub-screen

static const char *_weight_values[] = {
    "Regular",
    "Medium",
    "Semibold",
    NULL
};

static const char *_language_values[] = {
    "en",
    "ko",
    "ja",
    "hy",
    "he",
    NULL
};

static const char *_style_values[] = {
    "Regular",
    "Medium",
    "Bold",
    "Italic",
    NULL
};

static const char *_caps_values[] = {
    "otlayout\\:taml",
    "otlayout\\:telu",
    "otlayout\\:latn",
    "otlayout\\:khmr",
    NULL
};

// Create list view with given values array and callback
static void _create_list_view(appdata_s *ad, const char **labels, Evas_Smart_Cb callback)
{
    if (!ad) {
        dlog_print(DLOG_ERROR, LOG_TAG, "NULL appdate pointer received");
        return;
    }

    // List for possible font family values
    Evas_Object *current_list = elm_list_add(ad->win);

    if (!current_list) {
        // Add error handling (error message)
        dlog_print(DLOG_ERROR, LOG_TAG, "Failed to add new list");
        return;
    }

    elm_scroller_policy_set(current_list, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);
    evas_object_show(current_list);

    elm_naviframe_item_push(ad->navi, ad->app_name, NULL, NULL, current_list, NULL);

    // Create list with font matching values
    int i = 0;

    while (labels[i]) {
        elm_list_item_append(current_list, labels[i], NULL, NULL, callback, (void *)ad);
        i++;
    }

    elm_list_go(current_list);
    // Do not allow multi selection
    elm_list_multi_select_set(current_list, EINA_FALSE);
}

// Font weight selection
static void _weight_value_list_clicked(void *data, Evas_Object *obj, void *event_info)
{
    if (!data) {
        dlog_print(DLOG_ERROR, LOG_TAG, "NULL data pointer received");
        return;
    }

    appdata_s *ad = (appdata_s *)data;

    Elm_Object_Item *item = (Elm_Object_Item *)event_info;
    elm_list_item_selected_set(item, EINA_FALSE);
    const char *item_label = elm_object_item_text_get(item);

    show_fonts_by_weight(ad, item_label);
}

void create_font_weight_selection_screen(appdata_s *ad)
{
    dlog_print(DLOG_DEBUG, LOG_TAG, "Creating font weight selection screen");
    _create_list_view(ad, _weight_values, _weight_value_list_clicked);
}

// Font language selection
static void _language_value_list_clicked(void *data, Evas_Object *obj, void *event_info)
{
    if (!data) {
        dlog_print(DLOG_ERROR, LOG_TAG, "NULL data pointer received");
        return;
    }

    appdata_s *ad = (appdata_s *)data;

    Elm_Object_Item *item = (Elm_Object_Item *)event_info;
    elm_list_item_selected_set(item, EINA_FALSE);
    const char *item_label = elm_object_item_text_get(item);

    show_fonts_by_lang(ad, item_label);
}

void create_font_lang_selection_screen(appdata_s *ad)
{
    dlog_print(DLOG_DEBUG, LOG_TAG, "Creating font lang selection screen");
    _create_list_view(ad, _language_values, _language_value_list_clicked);
}

// Font size selection
static void _style_value_list_clicked(void *data, Evas_Object *obj, void *event_info)
{
    if (!data) {
        dlog_print(DLOG_ERROR, LOG_TAG, "NULL data pointer received");
        return;
    }

    appdata_s *ad = (appdata_s *)data;

    Elm_Object_Item *item = (Elm_Object_Item *)event_info;
    elm_list_item_selected_set(item, EINA_FALSE);
    const char *item_label = elm_object_item_text_get(item);

    show_fonts_by_style(ad, item_label);
}

void create_style_selection_screen(appdata_s *ad)
{
    dlog_print(DLOG_DEBUG, LOG_TAG, "Creating font style selection screen");
    _create_list_view(ad, _style_values, _style_value_list_clicked);
}

// Font width selection
static void _capability_value_list_clicked(void *data, Evas_Object *obj, void *event_info)
{
    if (!data) {
        dlog_print(DLOG_ERROR, LOG_TAG, "NULL data pointer received");
        return;
    }

    appdata_s *ad = (appdata_s *)data;

    Elm_Object_Item *item = (Elm_Object_Item *)event_info;
    elm_list_item_selected_set(item, EINA_FALSE);
    const char *item_label = elm_object_item_text_get(item);

    show_fonts_by_capability(ad, item_label);
}

void create_font_capability_selection_screen(appdata_s *ad)
{
    dlog_print(DLOG_DEBUG, LOG_TAG, "Creating font capability selection screen");
    _create_list_view(ad, _caps_values, _capability_value_list_clicked);
}
